"""Public key registration app."""

import json
from typing import Annotated

import typer
from rich.table import Table
from slices_pubkey_client.exceptions import ApiException, BadRequestException, NotFoundException
from slices_pubkey_client.models.public_key_create import PublicKeyCreate

from . import console, err_console
from .context import Context

pubkey_app = typer.Typer(name="pubkey", help="SSH key management", no_args_is_help=True)


@pubkey_app.command(name="register")
def register_pubkey(ctx: Context, file: Annotated[typer.FileText, typer.Argument]):
    """Register public key."""
    pubkey_api = ctx.obj.api.pubkey_api

    pubkey = file.read()

    PublicKeyCreate(public_key=pubkey)

    try:
        pubkey_api.register_user_pubkey(
            ctx.obj.get_logged_in_userinfo("sub"),
            PublicKeyCreate(public_key=pubkey),
        )
        console.print("[bold green]Registered public key.[/] :white_check_mark:")
    except BadRequestException as exc:
        err_console.print("[bold red]Could not register public key[/] :x:")
        err_console.print("[red]" + json.loads(exc.body)["detail"]) # type: ignore
        raise typer.Exit(code=1) from None
    except ApiException as exc:
        err_console.print("[bold red]Could not register public key[/] :x:")
        validation_errors = json.loads(exc.body)["detail"] # type: ignore
        for validation_error in validation_errors:
            err_console.print("[red]" + validation_error["msg"])
        raise typer.Exit(code=1) from None


@pubkey_app.command(name="list")
def list_pubkeys(ctx: Context):
    """List registered public key."""
    pubkey_api = ctx.obj.api.pubkey_api

    pubkeys = pubkey_api.get_user_pubkeys(ctx.obj.get_logged_in_userinfo("sub"))

    table = Table("ID", "Fingerprint")
    for pubkey in pubkeys.public_keys:
        table.add_row(str(pubkey.id), pubkey.sha256_fingerprint)

    console.print(table)
    console.print(f"{pubkeys.total} public keys registered")


@pubkey_app.command(name="delete")
def delete_pubkey(ctx: Context, id: int):
    """Delete registered public key."""

    pubkey_api = ctx.obj.api.pubkey_api
    try:
        pubkey = pubkey_api.delete_pubkey(id)

        console.print(f"Deleted key {pubkey.id} with fingerprint {pubkey.sha256_fingerprint}")
    except NotFoundException:
        err_console.print("[bold red]No public key with this ID. :x:")
